@@ -15,7 +15,7 @@
 # * name comes from http://www.thefreedictionary.com/cleave      #
 ##################################################################
 #                                                                # 
-# last changed in version = '2.1-1'                             #
+# last changed in version = '2.4-1'                              #
 #                                                                # 
 ##################################################################
 
@@ -55,6 +55,9 @@ fi
 if [ "$1" == 'getWlanInformation' ]; then 
     if [ -f $2/wlanInfo-01.csv ]; then rm $2/wlanInfo-* ; fi
     airodump-ng -w $2/wlanInfo wlan0 & sleep $3 && kill $!
+    if [ -f /usr/bin/walsh ]; then 
+       walsh -f $2/wlanInfo-01.cap -C -o /home/user/.reaver/wpsInfo.txt
+    fi
 fi 
 
 if [ "$1" == 'readESSIDs' ]; then
@@ -117,4 +120,22 @@ if [ "$1" == 'decryptWEP' ]; then
     aircrack-ng -l $2/keys/$3.txt $2/capturedData/WEP/$4
 fi 
 
-# if  [ "$1" == '' ]; then ; fi 
\ No newline at end of file
+if  [ "$1" == 'wpscrack' ]; then 
+    /opt/cleven/wpscrack.py --iface wlan0 --client $2 --bssid $3 --ssid $4 -v 
+fi 
+
+if  [ "$1" == 'reaver' ]; then 
+    reaver -i wlan0 -b $2 -c $3 -e $4 -a -E -vv 
+fi 
+
+if  [ "$1" == 'walsh' ]; then 
+    cat /home/user/.reaver/wpsInfo.txt | grep -e ":"
+fi 
+
+if  [ "$1" == 'copyReaver' ]; then 
+    cp -pr $2/reaver $3 && chmod 755 $3/reaver 
+fi 
+
+if  [ "$1" == 'copyWalsh' ]; then 
+    cp -pr $2/walsh $3 && chmod 755 $3/walsh
+    fi 
