@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.5
+#!/usr/bin/env python
 ##################################################################
 ##   Cleven  -  a user interface of aircrack-ng for the N900    ##
 ##################################################################
@@ -11,7 +11,7 @@
 # * name comes from http://www.thefreedictionary.com/cleave      #
 ##################################################################
 
-Version = '2.2'
+Version = '2.4'
 Build = '1'
 
 import sys
@@ -32,13 +32,16 @@ from clevenDictionaryUI import *
 from clevenKeysUI import *
 from clevenManageCapturedUI import *
 
-pathMain             = '/opt/cleven'                        # main path of files
-pathTemp             = '/home/user/.cleven'                 # temp path of files
+pathMain             = '/opt/cleven'                        # main path for the application
+pathStorage          = '/home/user/.cleven'                 # storage path for the application
 pathDrivers          = '%s/compat-wireless' % pathMain
+pathReaver           = '/home/user/.reaver'                 # storage path for reaver
 helperScript         = 'clevenHelper.sh'
 folderKeys           = 'keys'
 folderDictionaries   = 'dictionaries'
 folderCapturedData   = 'capturedData'
+reaverBinary         = 'reaver'
+walshBinary          = 'walsh'
 
 ##############################################################
 
@@ -123,6 +126,7 @@ class clevenCapture(QtGui.QMainWindow):
        QtCore.QObject.connect(self.ui.scanButton,  QtCore.SIGNAL('clicked()'), self.scanNetworks)
        QtCore.QObject.connect(self.ui.wepRadio,    QtCore.SIGNAL('clicked()'), self.displayNetworks)
        QtCore.QObject.connect(self.ui.wpaRadio,    QtCore.SIGNAL('clicked()'), self.displayNetworks)
+       QtCore.QObject.connect(self.ui.wpsRadio,    QtCore.SIGNAL('clicked()'), self.displayNetworks)
        QtCore.QObject.connect(self.ui.scanList,    QtCore.SIGNAL('itemSelectionChanged()'), self.displayNetworkInformation)
 
        ##Capture of packets
@@ -130,12 +134,20 @@ class clevenCapture(QtGui.QMainWindow):
        QtCore.QObject.connect(self.ui.authenticateButton,   QtCore.SIGNAL('clicked()'), self.executeAuthenticate)
        QtCore.QObject.connect(self.ui.deauthenticateButton, QtCore.SIGNAL('clicked()'), self.executeDeauthenticate)
        QtCore.QObject.connect(self.ui.injectionButton,      QtCore.SIGNAL('clicked()'), self.executeInjection)
+       QtCore.QObject.connect(self.ui.reaverButton,         QtCore.SIGNAL('clicked()'), self.executeReaver)
 
+       if reaverAvailability == False:
+          self.ui.reaverButton.setHidden(True)
+       if walshAvailability == False:
+          self.ui.wpsRadio.setHidden(True)
+       
        self.ui.captureButton.setEnabled(False)
+       self.ui.reaverButton.setEnabled(False)
+       
        self.ui.injectionButton.setHidden(True)
        self.ui.authenticateButton.setHidden(True)
        self.ui.deauthenticateButton.setHidden(True)
-      
+
        ##Show some info in the UI
        self.ui.textBrowser.clear()
        self.ui.textBrowser.append('Scan first for available networks')
@@ -159,7 +171,7 @@ class clevenCapture(QtGui.QMainWindow):
           message ='Scanning for available networks.\nPlease wait..'
           showQuickMessage(message) 
           
-          os.system('sudo %s/%s getWlanInformation %s %s' % (pathMain, helperScript, pathTemp, duration))
+          os.system('sudo %s/%s getWlanInformation %s %s' % (pathMain, helperScript, pathStorage, duration))
           message ='Scan for available networks completed!\nChoose WEP or WPA..'
           showQuickMessage(message)
 
@@ -176,6 +188,7 @@ class clevenCapture(QtGui.QMainWindow):
        self.ui.textBrowser.clear()
        self.ui.textBrowser.append('Select network from the list')
        self.ui.captureButton.setEnabled(False)
+       self.ui.reaverButton.setEnabled(False)
        self.ui.injectionButton.setHidden(True)
        self.ui.authenticateButton.setHidden(True)
        self.ui.deauthenticateButton.setHidden(True)
@@ -185,30 +198,45 @@ class clevenCapture(QtGui.QMainWindow):
        else:
           encryption = "WPA"      
 
-       values = os.popen('sudo %s/%s readESSIDs %s %s' %(pathMain, helperScript, pathTemp, encryption)).read()
-       values = values.lstrip(" ")            ##remove trailing space from first ESSID
-       values = values.replace("\n ","\n")    ##remove trailing space from the rest of the ESSIDs 
-       values = values.split("\n")            ##split them to create an entry for each in the list
-       # print "ESSIDs found:\n", values
-       values = filter(None, values)          ##remove unknown (hidden)
-       # print "ESSIDs cleaned:\n", values
-       values.sort(key=str.lower)             ##sort them by name.
-       # print "ESSIDs sorted:\n", values
+       if self.ui.wpsRadio.isChecked():
+          values = []
+          rValues = os.popen('sudo %s/%s walsh' %(pathMain, helperScript)).read()
+          rValues = rValues.split("\n")                     ##split them to create an entry for each in the list
+          for i in rValues:                                 ## iterate on the values stored in the list
+             values.append(i[18:].lstrip(" ").rstrip(" "))  ## trim the first 18 characters (i.e. the MAC) and spaces from left and right sides.
+          # print 'The WPS list is:\n',values
+       else:
+          values = os.popen('sudo %s/%s readESSIDs %s %s' %(pathMain, helperScript, pathStorage, encryption)).read()
+          values = values.lstrip(" ")                       ##remove trailing space from first ESSID
+          values = values.replace("\n ","\n")               ##remove trailing space from the rest of the ESSIDs 
+          values = values.split("\n")                       ##split them to create an entry for each in the list
+          # print 'The WEP/WPA list is:\n',values
 
-       # print "BSSIDs to put in the list:\n", values
+       values = filter(None, values)                        ##remove unknown (hidden)
+       values.sort(key=str.lower)                           ##sort them by name.
+       # print 'The final list is:\n',values
        self.ui.scanList.addItems(values)
 
     def displayNetworkInformation(self): #FIXME: read wlan info directly and not from a file.
 
        global essid, bssid, channel, encryption, power, essid_clean
-    
+
+       self.ui.captureButton.setEnabled(True)
+       self.ui.captureButton.setHidden(False)
+       if self.ui.wepRadio.isChecked() == False:
+          if reaverAvailability:
+             self.ui.reaverButton.setEnabled(True)
+             self.ui.reaverButton.setHidden(False)
+       
        if self.ui.scanList.currentRow() == -1:
           self.ui.textBrowser.clear()
           self.ui.captureButton.setEnabled(False)
+          self.ui.reaverButton.setEnabled(False)
           return
-       self.ui.captureButton.setEnabled(True)
+          
        essid = self.ui.scanList.currentItem().text()
-       essidInfo = os.popen('sudo %s/%s readESSIDinfo %s \"%s\"' % (pathMain, helperScript, pathTemp, essid)).read().split("\n")
+       # print '(taken from the list) ESSID: ',essid
+       essidInfo = os.popen('sudo %s/%s readESSIDinfo %s \"%s\"' % (pathMain, helperScript, pathStorage, essid)).read().split("\n")
        
        essid      = essidInfo[0]
        bssid      = essidInfo[1]
@@ -231,44 +259,59 @@ class clevenCapture(QtGui.QMainWindow):
        character_list = [' ', '(', ')', '!', '$', '%', '^', '&', '*', '-', '+', '=', '#', '@', ';', ':', '?', '/', '>', '<'] 
        for i in character_list:
           essid_clean = essid_clean.replace(i,'_')
-       print 'essid (clean): ', essid_clean   
+       print 'essid: ', essid, ' to essid (clean): ', essid_clean   
        
     def executeCapture(self):
        '''Opens an xterm window that captures the packets.'''
-   
-       if self.ui.scanList.currentRow() == -1:
+       
+       global encryption
+       
+       ## disable buttons again if no row is selected and return.   
+       if self.ui.scanList.currentRow() == -1: 
           self.ui.captureButton.setEnabled(False)
+          self.ui.reaverButton.setEnabled(False)
+
           self.ui.authenticateButton.setEnabled(False)
           self.ui.injectionButton.setEnabled(False)
           self.ui.deauthenticateButton.setEnable(False)
+
           self.ui.injectionButton.setHidden(True)
           self.ui.authenticateButton.setHidden(True)
           self.ui.deauthenticateButton.setHidden(True)
+
           return
+
+       ##Remove not needed buttons   
+       self.ui.captureButton.setHidden(True)
+       self.ui.captureButton.setEnabled(False)
+       self.ui.reaverButton.setHidden(True)
+       self.ui.reaverButton.setEnabled(False)
       
        if self.ui.wepRadio.isChecked():
           encryption = "WEP"
 
           self.ui.authenticateButton.setHidden(False)
           self.ui.injectionButton.setHidden(False)
-          self.ui.deauthenticateButton.setHidden(True)
+          self.ui.deauthenticateButton.setHidden(False)
 
           self.ui.authenticateButton.setEnabled(True)
           self.ui.injectionButton.setEnabled(True)
-          self.ui.deauthenticateButton.setEnabled(False)
+          self.ui.deauthenticateButton.setEnabled(True)
+
        else:
           encryption = "WPA"
+
           self.ui.authenticateButton.setHidden(True)
           self.ui.injectionButton.setHidden(True)
           self.ui.deauthenticateButton.setHidden(False)
-
+          
           self.ui.authenticateButton.setEnabled(False)
           self.ui.injectionButton.setEnabled(False)
           self.ui.deauthenticateButton.setEnabled(True)
           
-       # print "capture options: ", channel, bssid, pathTemp, encryption, essid, essid_clean
-       # os.system('osso-xterm -e "sudo %s/%s capture %s %s %s %s \"%s\" " &' % (pathMain, helperScript, channel, bssid, pathTemp, encryption, essid))
-       os.system('osso-xterm -e "sudo %s/%s capture %s %s %s %s \"%s\" " &' % (pathMain, helperScript, channel, bssid, pathTemp, encryption, essid_clean))
+       # print "capture options: ", channel, bssid, pathStorage, encryption, essid, essid_clean
+       # os.system('osso-xterm -e "sudo %s/%s capture %s %s %s %s \"%s\" " &' % (pathMain, helperScript, channel, bssid, pathStorage, encryption, essid))
+       os.system('osso-xterm -e "sudo %s/%s capture %s %s %s %s \"%s\" " &' % (pathMain, helperScript, channel, bssid, pathStorage, encryption, essid_clean))
       
     def executeDeauthenticate(self): #FIXME:: the essid should be the connected client's mac; atm ignoring essid and just broadcasting. 
        '''Opens an xterm window that sends deauthentication packets.'''
@@ -287,6 +330,12 @@ class clevenCapture(QtGui.QMainWindow):
        
        # print "injection options: ", bssid, myMAC
        os.system('osso-xterm -e "sudo %s/%s injection %s %s" &' % (pathMain, helperScript, bssid, myMAC))
+
+    def executeReaver(self):
+       '''Opens an xterm window that sends authentication packets.'''
+       
+       # print "reaver -i wlan0 -b ",bssid," -c ",channel," -e ",essid," -a -E -vv" 
+       os.system('osso-xterm -e "sudo %s/%s reaver %s %s %s" &' % (pathMain, helperScript, bssid, channel, essid))
        
 class clevenConfigureDrivers(QtGui.QDialog):
 
@@ -490,7 +539,7 @@ class clevenSelectDictionary(QtGui.QDialog):
        QtCore.QObject.connect(self.ui.dictionaryList, QtCore.SIGNAL('itemSelectionChanged()'), self.setDictionary)
        
        self.ui.dictionaryList.clear()     
-       dlist = os.listdir(pathTemp + '/' + folderDictionaries)
+       dlist = os.listdir(pathStorage + '/' + folderDictionaries)
        self.ui.dictionaryList.addItems(dlist)
        
     def setDictionary(self):
@@ -579,7 +628,7 @@ class clevenManageCaptured(QtGui.QMainWindow):
        self.ui.capturedList.clear() 
        self.ui.performActionButton.setEnabled(False)
        
-       clist = os.listdir(pathTemp + '/' + folderCapturedData + '/' + encryption)
+       clist = os.listdir(pathStorage + '/' + folderCapturedData + '/' + encryption)
        clist_clean = [line for line in clist if (('.cap' in line) or ('.ivs' in line))]
        clist_clean.sort(key=str.lower)
        self.ui.capturedList.addItems(clist_clean)
@@ -614,10 +663,10 @@ class clevenManageCaptured(QtGui.QMainWindow):
                 self.ui.performActionButton.setEnabled(False)
                 return
              else:
-                os.system('osso-xterm -e "sudo %s/%s decryptWPA %s %s %s %s" &' % (pathMain, helperScript, pathTemp, myDictionary, key, cap))
+                os.system('osso-xterm -e "sudo %s/%s decryptWPA %s %s %s %s" &' % (pathMain, helperScript, pathStorage, myDictionary, key, cap))
           else:
-             # print pathMain, pathTemp, key, cap
-             os.system('osso-xterm -e "sudo %s/%s decryptWEP %s %s %s" &' % (pathMain, helperScript, pathTemp, key, cap))
+             # print pathMain, pathStorage, key, cap
+             os.system('osso-xterm -e "sudo %s/%s decryptWEP %s %s %s" &' % (pathMain, helperScript, pathStorage, key, cap))
        
        ##Delete files             
        elif deleteModeOn == 'True':
@@ -636,7 +685,7 @@ class clevenManageCaptured(QtGui.QMainWindow):
              cap = self.capfile.strip().replace('.cap','').replace('.ivs','')
           print "delete cap: ", cap
 
-          os.system('sudo %s/%s deleteCaptured %s %s %s' % (pathMain, helperScript, pathTemp, encryption, cap))
+          os.system('sudo %s/%s deleteCaptured %s %s %s' % (pathMain, helperScript, pathStorage, encryption, cap))
           self.showCapturedFiles()          
          
 class clevenShowKeys(QtGui.QMainWindow):
@@ -664,7 +713,7 @@ class clevenShowKeys(QtGui.QMainWindow):
 
     def refreshKeyList(self): 
     
-       list = os.listdir(pathTemp + '/' + folderKeys)
+       list = os.listdir(pathStorage + '/' + folderKeys)
        self.ui.keyList.clear()
        self.ui.keyList.addItems(list)
        self.ui.keyBrowser.setText("Please select one of the Key files..")
@@ -676,7 +725,7 @@ class clevenShowKeys(QtGui.QMainWindow):
           return
        keyfile = str(self.ui.keyList.currentItem().text()).strip()
        try:
-          key = open(pathTemp + '/' + folderKeys + '/' + keyfile, 'r' ).readline()
+          key = open(pathStorage + '/' + folderKeys + '/' + keyfile, 'r' ).readline()
           self.ui.keyBrowser.setText(key)
        except IOError:
           self.ui.keyBrowser.setText("Key file cannot be read..")
@@ -774,7 +823,7 @@ class clevenStartup(QtGui.QDialog):
        self.step = self.step + 1
        self.ui.progressBarStart.setValue(self.step)
     
-class clevenMain():
+class clevenMain(): ## TODO: add a check for reaver directory
 
     ##Check with what priviledges the GUI has been executed.
     if os.geteuid() == 0:
@@ -782,19 +831,44 @@ class clevenMain():
        showMessage(message)
        sys.exit(1)
     
-    ##Check if temp folder is available   
-    if os.path.exists(pathTemp) == False:
-       message ='First Time Use: Creating user folders at:\n%s' % pathTemp
+    ##Check if user's storage folder is available:   
+    if os.path.exists(pathStorage) == False:
+       message ='First Time Use: Creating user folders at:\n%s' % pathStorage
        showMessage(message)    
-       os.system('mkdir -p %s' % pathTemp)
-       os.system('mkdir -p %s/%s' % (pathTemp,folderCapturedData))
-       os.system('mkdir -p %s/%s/WEP' % (pathTemp,folderCapturedData))
-       os.system('mkdir -p %s/%s/WPA' % (pathTemp,folderCapturedData))
-       os.system('mkdir -p %s/%s' % (pathTemp,folderDictionaries))
-       os.system('mkdir -p %s/%s' % (pathTemp,folderKeys))
+       os.system('mkdir -p %s'         % (pathStorage))
+       os.system('mkdir -p %s/%s'      % (pathStorage,folderCapturedData))
+       os.system('mkdir -p %s/%s/WEP'  % (pathStorage,folderCapturedData))
+       os.system('mkdir -p %s/%s/WPA'  % (pathStorage,folderCapturedData))
+       os.system('mkdir -p %s/%s'      % (pathStorage,folderDictionaries))
+       os.system('mkdir -p %s/%s'      % (pathStorage,folderKeys))
     else:
        pass
     
+    global reaverAvailability, walshAvailability
+    reaverAvailability = False
+    walshAvailability = False
+    
+    ##Check if reaver and/or walsh is available in the home directory:
+    if os.path.exists(pathReaver):
+       if os.path.isfile(pathReaver + '/' + reaverBinary):
+          if os.WEXITSTATUS(os.system('sudo %s/%s copyReaver %s %s' % ( pathMain, helperScript, pathReaver, pathMain ))) == 0:
+             print 'found Reaver binary and made a copy'
+       if os.path.isfile(pathReaver + '/' + walshBinary):
+          if os.WEXITSTATUS(os.system('sudo %s/%s copyWalsh %s %s' % ( pathMain, helperScript, pathReaver, pathMain ))) == 0:
+             print 'found Walsh binary and made a copy'
+
+    ##Check if reaver is available to the application:
+    if os.path.isfile(pathMain + '/' + reaverBinary):
+       os.path.exists(pathReaver) or os.mkdir(pathReaver)
+       reaverAvailability = True
+       print 'Reaver is available for usage..'
+       
+    ##Check if walsh is available to the application:
+    if os.path.isfile(pathMain + '/' + walshBinary):
+       os.path.exists(pathReaver) or os.mkdir(pathReaver)
+       walshAvailability = True
+       print 'Walsh is available for usage..'
+    
     app = QtGui.QApplication(sys.argv)
     myapp = clevenMainWindow()
     myapp.show()
